package com.screentimetracker.service

import android.app.KeyguardManager
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.os.PowerManager
import android.util.Log
import kotlinx.coroutines.*

class ScreenStateManager(
    private val context: Context,
    private val timeTracker: ScreenTimeTracker,
    private val scope: CoroutineScope
) {
    companion object {
        private const val TAG = "ScreenStateManager"
        private const val WAKE_LOCK_TAG = "ScreenTimeService:WakeLock"
        private const val SCREEN_OFF_RESET_DELAY = 30 // 30 secondes
    }

    private lateinit var wakeLock: PowerManager.WakeLock
    private lateinit var keyguardManager: KeyguardManager
    private var lockTimer: Job? = null
    private var isScreenOn = true

    private val screenReceiver = object : BroadcastReceiver() {
        override fun onReceive(context: Context?, intent: Intent?) {
            when (intent?.action) {
                Intent.ACTION_SCREEN_OFF -> handleScreenOff()
                Intent.ACTION_SCREEN_ON -> handleScreenOn()
                Intent.ACTION_USER_PRESENT -> handleUserPresent()
            }
        }
    }

    fun initialize() {
        val powerManager = context.getSystemService(Context.POWER_SERVICE) as PowerManager
        keyguardManager = context.getSystemService(Context.KEYGUARD_SERVICE) as KeyguardManager

        wakeLock = powerManager.newWakeLock(PowerManager.PARTIAL_WAKE_LOCK, WAKE_LOCK_TAG)
        wakeLock.acquire(10*60*1000L)

        val filter = IntentFilter().apply {
            addAction(Intent.ACTION_SCREEN_ON)
            addAction(Intent.ACTION_SCREEN_OFF)
            addAction(Intent.ACTION_USER_PRESENT)
        }
        context.registerReceiver(screenReceiver, filter)
    }

    private fun handleScreenOff() {
        Log.d(TAG, "Écran éteint")
        isScreenOn = false
        if (wakeLock.isHeld) {
            wakeLock.release()
        }
        startLockTimer()
    }

    private fun handleScreenOn() {
        Log.d(TAG, "Écran allumé")
        isScreenOn = true
        if (!wakeLock.isHeld) {
            wakeLock.acquire(10*60*1000L)
        }
    }

    private fun handleUserPresent() {
        Log.d(TAG, "Appareil déverrouillé")
        lockTimer?.cancel()
    }

    private fun startLockTimer() {
        lockTimer?.cancel()
        lockTimer = scope.launch {
            delay(SCREEN_OFF_RESET_DELAY * 1000L)
            if (keyguardManager.isKeyguardLocked) {
                Log.d(TAG, "Réinitialisation du temps continu après 30 secondes de verrouillage")
                timeTracker.resetContinuousTime()
            }
        }
    }

    fun isDeviceActive(): Boolean {
        return isScreenOn && !keyguardManager.isKeyguardLocked
    }

    fun cleanup() {
        if (wakeLock.isHeld) {
            wakeLock.release()
        }
        lockTimer?.cancel()
        context.unregisterReceiver(screenReceiver)
    }
}