package com.screentimetracker.service

import android.content.Context
import android.graphics.PixelFormat
import android.os.Handler
import android.os.Looper
import android.provider.Settings
import android.util.Log
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.WindowManager
import android.widget.TextView
import android.content.res.Configuration
import com.soberdigital.screentimetracker.R

class ScreenTimeOverlayManager(
    private val context: Context,
    private val windowManager: WindowManager
) {
    companion object {
        private const val TAG = "ScreenTimeOverlay"
        private const val PORTRAIT_TOP_MARGIN_DP = 20
        private const val LANDSCAPE_TOP_MARGIN_DP = 20
        private const val PORTRAIT_WIDTH_PERCENT = 0.7f
        private const val LANDSCAPE_WIDTH_PERCENT = 0.5f
    }

    private var overlayView: View? = null
    private val mainHandler = Handler(Looper.getMainLooper())
    private var isOverlayVisible = false
    private var currentParams: WindowManager.LayoutParams? = null
    private var onDismissListener: (() -> Unit)? = null

    fun setOnDismissListener(listener: () -> Unit) {
        onDismissListener = listener
    }

    private fun dpToPx(dp: Int): Int {
        val density = context.resources.displayMetrics.density
        return (dp * density).toInt()
    }

    private fun getScreenWidth(): Int {
        return context.resources.displayMetrics.widthPixels
    }

    private fun createWindowParams(): WindowManager.LayoutParams {
        val screenWidth = getScreenWidth()
        val isPortrait = context.resources.configuration.orientation == Configuration.ORIENTATION_PORTRAIT
        val widthPercent = if (isPortrait) PORTRAIT_WIDTH_PERCENT else LANDSCAPE_WIDTH_PERCENT
        val topMargin = if (isPortrait) PORTRAIT_TOP_MARGIN_DP else LANDSCAPE_TOP_MARGIN_DP
        val overlayWidth = (screenWidth * widthPercent).toInt()

        return WindowManager.LayoutParams().apply {
            width = overlayWidth
            height = WindowManager.LayoutParams.WRAP_CONTENT
            type = WindowManager.LayoutParams.TYPE_APPLICATION_OVERLAY
            flags = WindowManager.LayoutParams.FLAG_NOT_FOCUSABLE or
                    WindowManager.LayoutParams.FLAG_NOT_TOUCH_MODAL
            format = PixelFormat.TRANSLUCENT
            gravity = Gravity.TOP or Gravity.CENTER_HORIZONTAL
            y = dpToPx(topMargin)
        }
    }

    fun showOverlay(continuousScreenTimeSeconds: Int) {
        Log.d(TAG, "Tentative d'affichage de l'overlay - Temps continu: $continuousScreenTimeSeconds secondes")

        if (!Settings.canDrawOverlays(context)) {
            Log.e(TAG, "Permission SYSTEM_ALERT_WINDOW non accordée")
            return
        }

        mainHandler.post {
            try {
                if (isOverlayVisible) {
                    updateOverlayText(continuousScreenTimeSeconds)
                    updateOverlayPosition()
                    return@post
                }

                Log.d(TAG, "Début de la création de l'overlay")

                overlayView = LayoutInflater.from(context).inflate(R.layout.overlay_bubble, null)
                updateOverlayText(continuousScreenTimeSeconds)

                currentParams = createWindowParams()

                overlayView?.setOnClickListener {
                    Log.d(TAG, "Clic sur l'overlay - Fermeture")
                    onDismissListener?.invoke()
                    removeOverlay()
                }

                windowManager.addView(overlayView, currentParams)
                isOverlayVisible = true
                Log.d(TAG, "Overlay ajouté avec succès")
            } catch (e: Exception) {
                Log.e(TAG, "Erreur lors de l'affichage de l'overlay", e)
                e.printStackTrace()
                removeOverlay()
            }
        }
    }

    private fun updateOverlayText(continuousScreenTimeSeconds: Int) {
        mainHandler.post {
            try {
                overlayView?.findViewById<TextView>(R.id.overlay_text)?.let { textView ->
                    val minutes = continuousScreenTimeSeconds / 60
                    textView.text = context.getString(R.string.continuous_screen_time_format, minutes)
                    Log.d(TAG, "Texte de l'overlay mis à jour")
                }
            } catch (e: Exception) {
                Log.e(TAG, "Erreur lors de la mise à jour du texte de l'overlay", e)
                e.printStackTrace()
            }
        }
    }

    private fun updateOverlayPosition() {
        mainHandler.post {
            try {
                overlayView?.let { view ->
                    currentParams = createWindowParams()
                    windowManager.updateViewLayout(view, currentParams)
                    Log.d(TAG, "Position de l'overlay mise à jour")
                }
            } catch (e: Exception) {
                Log.e(TAG, "Erreur lors de la mise à jour de la position de l'overlay", e)
                e.printStackTrace()
            }
        }
    }

    fun removeOverlay() {
        mainHandler.post {
            try {
                overlayView?.let {
                    Log.d(TAG, "Suppression de l'overlay")
                    windowManager.removeView(it)
                    overlayView = null
                    isOverlayVisible = false
                    currentParams = null
                    Log.d(TAG, "Overlay supprimé avec succès")
                }
            } catch (e: Exception) {
                Log.e(TAG, "Erreur lors de la suppression de l'overlay", e)
                e.printStackTrace()
            }
        }
    }
}