package com.screentimetracker.service

import android.content.Context
import android.content.Intent
import android.util.Log
import kotlinx.coroutines.*
import java.util.Calendar
import java.time.LocalDateTime
import java.time.LocalTime

class ScreenTimeTracker(
    private val context: Context,
    private val notificationManager: ScreenTimeNotificationManager,
    private val overlayManager: ScreenTimeOverlayManager,
    private val storage: ScreenTimeStorage,
    private val scope: CoroutineScope
) {
    companion object {
        private const val TAG = "ScreenTimeTracker"
        private const val CONTINUOUS_SCREEN_TIME_INTERVAL = 300 // 5 minutes en secondes
        private const val OVERLAY_UPDATE_INTERVAL = 60 // 1 minute en secondes
        const val ACTION_TIME_UPDATED = "com.screentimetracker.TIME_UPDATED"
        const val EXTRA_TIME = "extra_time"
    }

    private var screenTimeMinutes: Long = 0
    private var secondsCounter = 0
    private var continuousScreenTimeSeconds = 0
    private var isOverlayDismissed = false
    private var lastOverlayShowTime = 0
    private var lastDayOfYear = Calendar.getInstance().get(Calendar.DAY_OF_YEAR)

    init {
        loadSavedTime()
        overlayManager.setOnDismissListener { onOverlayDismissed() }
    }

    private fun loadSavedTime() {
        val data = storage.loadScreenTime()
        screenTimeMinutes = data.screenTimeMinutes
        secondsCounter = data.secondsCounter
        continuousScreenTimeSeconds = data.continuousScreenTimeSeconds
    }

    fun updateTime() {
        val calendar = Calendar.getInstance()
        val currentDayOfYear = calendar.get(Calendar.DAY_OF_YEAR)

        // Vérifier si nous avons changé de jour
        if (currentDayOfYear != lastDayOfYear) {
            resetDailyTime()
            lastDayOfYear = currentDayOfYear
        }

        secondsCounter++
        continuousScreenTimeSeconds++

        if (secondsCounter >= 60) {
            screenTimeMinutes++
            secondsCounter = 0
            notificationManager.updateNotification(screenTimeMinutes)
            broadcastCurrentTime()
            saveCurrentTime()
        }

        // Vérifier si nous devons afficher ou mettre à jour l'overlay
        if (continuousScreenTimeSeconds >= CONTINUOUS_SCREEN_TIME_INTERVAL) {
            if (!isOverlayDismissed) {
                // Si l'overlay n'a pas été fermé, le mettre à jour chaque minute
                if (continuousScreenTimeSeconds % OVERLAY_UPDATE_INTERVAL == 0) {
                    overlayManager.showOverlay(continuousScreenTimeSeconds)
                }
            } else {
                // Si l'overlay a été fermé, le réafficher tous les 5 minutes
                if (continuousScreenTimeSeconds % CONTINUOUS_SCREEN_TIME_INTERVAL == 0) {
                    isOverlayDismissed = false
                    overlayManager.showOverlay(continuousScreenTimeSeconds)
                }
            }
        }
    }

    private fun resetDailyTime() {
        screenTimeMinutes = 0
        secondsCounter = 0
        // Ne pas réinitialiser continuousScreenTimeSeconds
        notificationManager.updateNotification(screenTimeMinutes)
        broadcastCurrentTime()
        saveCurrentTime()
        Log.d(TAG, "Temps d'écran journalier réinitialisé")
    }

    private fun onOverlayDismissed() {
        isOverlayDismissed = true
        lastOverlayShowTime = continuousScreenTimeSeconds
    }

    private fun saveCurrentTime() {
        storage.saveScreenTime(
            ScreenTimeData(
                screenTimeMinutes,
                secondsCounter,
                continuousScreenTimeSeconds
            )
        )
    }

    fun resetContinuousTime() {
        continuousScreenTimeSeconds = 0
        isOverlayDismissed = false
        lastOverlayShowTime = 0
        overlayManager.removeOverlay()
        saveCurrentTime()
    }

    fun broadcastCurrentTime() {
        val intent = Intent(ACTION_TIME_UPDATED).apply {
            putExtra(EXTRA_TIME, screenTimeMinutes)
            setPackage(context.packageName)
        }
        context.sendBroadcast(intent)
    }

    fun cleanup() {
        saveCurrentTime()
        overlayManager.removeOverlay()
    }
}